@@ -10,8 +10,8 @@
 
 __author__ = """Stephen Wasilewski"""
 __email__ = 'stephanwaz@gmail.com'
-__version__ = '0.2.3'
-__all__ = ['io', 'skycalc', 'translate', 'draw', 'metric', 'lightfield',
+__version__ = '0.2.4'
+__all__ = ['io', 'skycalc', 'translate', 'draw', 'metricfuncs', 'lightfield',
            'integrator', 'mapper', 'sampler', 'scene', 'renderer']
 
 
