@@ -13,10 +13,12 @@ import subprocess
 
 import numpy as np
 import raytraverse
-from raytraverse import translate, io, skycalc, metric
+from raytraverse import translate, io, skycalc, metricfuncs
 from raytraverse.mapper import ViewMapper
 from raytraverse.scene import SkyInfo
 from raytraverse.crenderer import cRtrace
+from raytraverse.lightfield import SunSkyPt
+
 
 class Integrator(object):
     """class to generate outputs from skyfield sunfield and sky conditions
@@ -53,7 +55,7 @@ class Integrator(object):
     """
 
     def __init__(self, skyfield, sunfield=None, wea=None, loc=None, skyro=0.0,
-                 **kwargs):
+                 ground_fac=0.15, **kwargs):
         #: raytraverse.scene.Scene
         self.scene = skyfield.scene
         try:
@@ -75,6 +77,7 @@ class Integrator(object):
         self.sunfield = sunfield
         #: raytraverse.lightfield.SCBinField
         self.skyfield = skyfield
+        self.ground_fac = ground_fac
         if wea is not None:
             self.skydata = wea
         else:
@@ -143,7 +146,7 @@ class Integrator(object):
                              '\n5 col: m, d, h, dir, diff')
         return skydat
 
-    def get_sky_mtx(self, skydata=None):
+    def get_sky_mtx(self, skydata=None, ground_fac=None):
         """generate sky, grnd and sun coefficients from sky data using perez
 
         Parameters
@@ -171,6 +174,10 @@ class Integrator(object):
         daysteps: np.array
             shape (len(skydata),) boolean array masking timesteps when sun is
             below horizon
+        serrs: np.array
+            error (angle in degrees) between sun and proxy source
+        skydata: no.array
+            sun position and dirnorm diffhoriz
         """
         if skydata is None:
             skydata = self.skydata
@@ -179,22 +186,26 @@ class Integrator(object):
         daysteps = skydata[:, 2] > 0
         sxyz = skydata[daysteps, 0:3]
         if self.suns is not None:
-            si = self.suns.proxy_src(sxyz, tol=self.suns.sunres)
+            si, serrs = self.suns.proxy_src(sxyz, tol=self.suns.sunres)
             # nosun = np.arange(hassun.size)[np.logical_not(hassun)]
         else:
             # nosun = np.arange(sxyz.shape[0])
             # hassun = 0
-            si = np.zeros(sxyz.shape[0])
+            si = np.zeros(sxyz.shape[0], dtype=int)
+            serrs = si
         sunuv = translate.xyz2uv(sxyz, flipu=False)
         sunbins = translate.uv2bin(sunuv, self.scene.skyres)
         dirdif = skydata[daysteps, 3:]
-        smtx, grnd, sun = skycalc.sky_mtx(sxyz, dirdif, self.scene.skyres)
+        if ground_fac is None:
+            ground_fac = self.ground_fac
+        smtx, grnd, sun = skycalc.sky_mtx(sxyz, dirdif, self.scene.skyres,
+                                          ground_fac=ground_fac)
         # ratio between actual solar disc and patch
         omegar = np.square(0.2665 * np.pi * self.scene.skyres / 180) * .5
         plum = sun[:, -1] * omegar
         sun = np.hstack((sun, plum[:, None]))
         si = np.stack((si, sunbins)).T
-        return smtx, grnd, sun, si, daysteps
+        return smtx, grnd, sun, si, daysteps, serrs, skydata
 
     def header(self):
         """generate image header string"""
@@ -218,190 +229,229 @@ class Integrator(object):
             pass
         return hdr
 
-    def _hdr(self, res, vm, keymap, pi, suni, sun, pdirs, si, vstr, outf,
-             interp, mask, skyv):
-        img = np.zeros((res*vm.aspect, res))
-        if keymap[pi, suni[0]] and sun[-1] > 0:
-            sun = sun[0:4]
-            psi = (pi, suni[0])
-            j, e = self.sunfield.query_ray(psi, pdirs[mask],
-                                           interp=interp)
-            self.sunfield.add_to_img(img, mask, psi, j, e, sun[-1], vm)
+    def _get_metric_info(self, sunskyfield, skyv, sun, suni, pi, vm):
+        if suni[0] in sunskyfield.items() and sun[-2] > 0:
+            lf = sunskyfield
+            li = suni[0]
+            skyvec = np.concatenate((sun[-2:-1], skyv))
+            sunview = lf.view.get_ray((pi, li), vm, sun[0:4])
+            sunvalue = 1 + (sunview is not None)
         else:
-            skyv[suni[1]] += sun[4]
-        self.skyfield.add_to_img(img, mask, pi, *si, coefs=skyv)
-        io.array2hdr(img, outf, self.header() + [vstr])
-        return outf
+            lf = self.skyfield
+            li = pi
+            skyvec = np.copy(skyv)
+            skyvec[suni[1]] += sun[-1]
+            sunview = None
+            sunvalue = 0
+        return lf, li, skyvec, sunview, sunvalue
 
-    def hdr(self, pts, smtx, grnd=None, suns=None, suni=None,
-            daysteps=None, vname='view', viewangle=180.0, res=400, interp=1):
+    def hdr(self, sunskyfield, skyv, sun, suni, pi, vm, pdirs, mask, vstr,
+            outf, interp=1):
+        """interpolate and write hdr image for a single skyv/sun/pt-index
+        combination
+
+        Parameters
+        ----------
+        sunskyfield: raytraverse.lightfield.sunskypt
+        skyv: np.array
+            sky patch coefficients, size=self.skyfield.srcn
+            (last value is ground)
+        sun: np.array
+            sun value (dx, dy, dz, true radiance, patch radiance)
+        suni: tuple
+            (sun index, sun bin) index is position in self.suns.suns, bin is
+            corresponding patch in skyfield
+        pi: int
+            point index
+        vm: raytraverse.mapper.viewmapper
+            should have a view angle of 180 degrees, the analyis direction
+        pdirs: np.array
+            pixel ray directions, shape (res, res, 3)
+        mask: tuple
+            pair of integer np.array representing pixel coordinates of images
+            to calculate
+        vstr: str
+            view string for radiance compatible header
+        outf: str
+            destination file path
+        interp: int
+            number of rays to search for in query, interpolation always happens
+            between 3 points, but in order to find a valid mesh triangle more
+            than 3 points is typically necessary. 16 seems to be a safe number
+            set to 1 (default) to turn off interpolation and use nearest ray
+            this will result in voronoi patches in the final image.
+
+        Returns
+        -------
+        outf: str
+            saved output file
+        returntype: str
+            'hdr' indicating format of result (useful when called
+            as parallel process to seperate from 'metric' or other outputs)
+
+        """
+        img = np.zeros(pdirs.shape[:-1])
+        lf, li, skyvec, _, _ = self._get_metric_info(sunskyfield, skyv, sun,
+                                                     suni, pi, vm)
+        lf.add_to_img(img, mask, li, pdirs[mask], coefs=skyvec, vm=vm,
+                      interp=interp)
+        io.array2hdr(img, outf, self.header() + [vstr])
+        return outf, 'hdr'
+
+    def metric(self, sunskyfield, skyv, sun, suni, pi, vm, info,
+               mfuncs=(metricfuncs.illum,), **kwargs):
+        """calculate metrics for a single skyv/sun/pt-index combination
+
+        Parameters
+        ----------
+        sunskyfield: raytraverse.lightfield.sunskypt
+        skyv: np.array
+            sky patch coefficients, size=self.skyfield.srcn
+            (last value is ground)
+        sun: np.array
+            sun value (dx, dy, dz, true radiance, patch radiance)
+        suni: tuple
+            (sun index, sun bin) index is position in self.suns.suns, bin is
+            corresponding patch in skyfield
+        pi: int
+            point index
+        vm: raytraverse.mapper.viewmapper
+            should have a view angle of 180 degrees, the analyis direction
+        mfuncs: tuple
+            tuple of callables with the signature:
+            ``f(vm, rays, omega, lum, **kwargs)``
+            return a float
+        kwargs:
+            passed to metricfuncs
+
+        Returns
+        -------
+        data: np.array
+            results for skyv and pi, shape (len(info[0]) + len(metricfuncs) +
+            1 + len(info[1])
+        returntype: str
+            'metric' indicating format of result (useful when called
+            as parallel process to seperate from 'hdr' or other outputs)
+        """
+        lf, li, skyvec, svw, svl = self._get_metric_info(sunskyfield, skyv, sun,
+                                                         suni, pi, vm)
+        idx = lf.query_ball(li, vm.dxyz)[0]
+        omega = np.squeeze(lf.omega[li][idx])
+        rays = lf.vec[pi][idx]
+        lum = np.squeeze(lf.apply_coef(li, skyvec))[idx]
+        if svw is not None:
+            rays = np.vstack((rays, svw[0][None, :]))
+            lum = np.concatenate((lum, [svw[1]]))
+            omega = np.concatenate((omega, [svw[2]]))
+        fmetric = [f(vm, rays, omega, lum, **kwargs)
+                   for f in mfuncs]
+        data = np.concatenate((info[0], fmetric, [svl, ], info[1]))
+        return data, 'metric'
+
+    def integrate(self, pts, smtx, grnd=None, suns=None, suni=None,
+                  daysteps=None, sunerrs=None, skydata=None, dohdr=True,
+                  dometric=True, vname='view', viewangle=180.0, res=400,
+                  interp=1, mfuncs=(metricfuncs.illum,),
+                  metric_return_opts=None, **kwargs):
         """
 
         Parameters
         ----------
-        pts: np.array
-            points and view dirs shape (N, 6)
-        smtx: np.array
-            shape (len(skydata), skyres**2) coefficients for each sky patch
-            each row is a timestep, timesteps where a sun exists exclude the
-            sun coefficient, otherwise the patch enclosing the sun position
-            contains the energy of the sun
-        grnd: np.array
-            shape (len(skydata),) coefficients for ground at each timestep
-        suns: np.array
-            shape (len(skydata), 5) sun position (index 0,1,2) and coefficients
-            for sun at each timestep assuming the true solid angle of the sun
-            (index 3) and the weighted value for the sky patch (index 4).
-        suni: np.array
-             shape (len(skydata), 2) index for self.suns.suns pointing to
-             correct proxy source (col 0) and sunbin for patch mapping (col 1)
-        daysteps: np.array
-            shape (len(skydata),) boolean array masking timesteps when sun is
-            below horizon
-        vname: str
-            view name for output file
-        viewangle: float, optional
-            degree opening of view cone
-        res: int, optional
-            image resolution
-        interp: int, optional
-            number of nearest points to interpolate between. 1 will resemble
-            voronoi patches
+        pts
+        smtx
+        grnd
+        suns
+        suni
+        daysteps
+        sunerrs
+        skydata
+        dohdr
+        dometric
+        vname
+        viewangle
+        res
+        interp
+        mfuncs
+        metric_return_opts: dict
+            boolean dictionary of columns to print with metric output. Default:
+            {"idx": False, "sensor": False, False, "sky": False}
+        kwargs
 
         Returns
         -------
 
         """
-        npts = self.scene.area.npts
+
+        metricreturn = {"idx": False, "sensor": False,
+                        "err": False, "sky": False}
+        if metric_return_opts is not None:
+            metricreturn.update(metric_return_opts)
+        metricreturn["metric"] = True
         perrs, pis = self.scene.area.pt_kd.query(pts[:, 0:3])
-        if self.suns:
-            keymap = self.sunfield.keymap()
-        else:
-            keymap = np.full((npts, 1), False)
+        sort = np.argsort(pis)
+        s_pis = pis[sort]
+        s_perrs = perrs[sort]
+        s_pts = pts[sort]
+        if grnd is not None:
+            smtx = np.hstack((smtx, grnd[:, None]))
         with ProcessPoolExecutor(io.get_nproc()) as exc:
             fu = []
-            for pj, (pi, vdir) in enumerate(zip(pis, pts[:, 3:6])):
+            last_pi = None
+            sunsky = None
+            for pj, (pi, pt, vdir, perr) in enumerate(zip(s_pis, s_pts[:, 0:3],
+                                                          s_pts[:, 3:6],
+                                                          s_perrs)):
+                if pi != last_pi:
+                    sunsky = SunSkyPt(self.skyfield, self.sunfield, pi)
                 vm = ViewMapper(viewangle=viewangle, dxyz=vdir, name=vname)
+                vmm = ViewMapper(viewangle=180, dxyz=vdir)
                 vp = self.scene.area.idx2pt([pi])[0]
                 vstr = ('VIEW= -vta -vv {0} -vh {0} -vd {1} {2} {3}'
                         ' -vp {4} {5} {6}'.format(viewangle, *vdir, *vp))
                 pdirs = vm.pixelrays(res)
                 mask = vm.in_view(pdirs)
-                si = self.skyfield.query_ray(pi, pdirs[mask], interp=interp)
-                for sj, skyv in enumerate(smtx):
-                    outf = (f"{self.scene.outdir}_{vname}_{pj:04d}_{sj:04d}"
-                            ".hdr")
-                    fu.append(exc.submit(self._hdr, res, vm, keymap, pi,
-                                         suni[sj], suns[sj], pdirs, si, vstr,
-                                         outf, interp, mask, skyv))
-            for future in as_completed(fu):
-                print(future.result(), file=sys.stderr)
-
-    def metric(self, pts, smtx, grnd=None, suns=None, suni=None,
-               daysteps=None, metricfuncs=(metric.illum,), **kwargs):
-        """calculate luminance based metrics for given sensors and skyvecs
-
-        Parameters
-        ----------
-        pts: np.array
-            points and view dirs shape (N, 6)
-        smtx: np.array
-            shape (len(skydata), skyres^2) coefficients for each sky patch
-            each row is a timestep, timesteps where a sun exists exclude the
-            sun coefficient, otherwise the patch enclosing the sun position
-            contains the energy of the sun
-        grnd: np.array
-            shape (len(skydata),) coefficients for ground at each timestep
-        suns: np.array
-            shape (len(skydata), 5) sun position (index 0,1,2) and coefficients
-            for sun at each timestep assuming the true solid angle of the sun
-            (index 3) and the weighted value for the sky patch (index 4).
-        suni: np.array
-             shape (len(skydata), 2) index for self.suns.suns pointing to
-             correct proxy source (col 0) and sunbin for patch mapping (col 1)
-        daysteps: np.array
-            shape (len(skydata),) boolean array masking timesteps when sun is
-            below horizon
-        metricfuncs: tuple
-            list of metric functions to apply, with the call signature:
-            f(view, rays, omegas, lum, kwargs)
-
-        Returns
-        -------
-        metrics: np.array
-            metrics at each point/direction and sky weighting axes ordered as:
-            (view, points, skys, metrics) an additional column on the final axis
-            is 0 when only the skyfield is used, 1 when sun reflections exists
-            and 2 when a view to the sun also exists.
-        """
-        npts = self.scene.area.npts
-        perrs, pis = self.scene.area.pt_kd.query(pts[:, 0:3])
-        if self.suns:
-            keymap = self.sunfield.keymap()
-        else:
-            keymap = np.full((npts, 1), False)
-        pmetrics = []
-        for pi, vdir in zip(pis, pts[:, 3:6]):
-            # get sky vectors
-            skyidx = self.skyfield.query_ball(pi, [vdir])
-            smetrics = []
-            vm = ViewMapper(vdir, viewangle=180)
-            for skyv, s, (si, sb) in zip(smtx, suns, suni):
-                psi = (pi, si)
-                if keymap[pi, si] and s[-2] > 0:
-                    skyvec = skyv
-                    sunidx = self.sunfield.query_ball(psi, [vdir])
-                    sunlm = np.squeeze(self.sunfield.apply_coef(psi, s[-2]))
-                else:
-                    skyvec = np.copy(skyv)
-                    skyvec[sb] += s[-1]
-                    sunidx = False
-                    sunlm = False
-                skylm = np.squeeze(self.skyfield.apply_coef(pi, skyvec))
-                skyrays = self.skyfield.vec[pi][skyidx[0]]
-                osky = np.squeeze(self.skyfield.omega[pi][skyidx[0]])
-                lmsky = skylm[skyidx[0]]
-                sunvalue = 0
-                mstack = []
-                skyweight = 1
-                if sunidx:
-                    sunvalue += 1
-                    sunrays = self.sunfield.vec[psi][sunidx[0]]
-                    osun = np.squeeze(self.sunfield.omega[psi][sunidx[0]])
-                    lmsun = sunlm[sunidx[0]]
-
-                    # TODO cross match samples
-                    # currently implements a nearest neighbor and then
-                    # averages the two results (using the skyweight term,
-                    # this is subject to some error a better approach is
-                    # to query the ray for neighbors with a radius derived
-                    # from omega and average in the case of no result
-                    # (when omega_other is larger) use the nearest neighbor.
-                    # This will likely need to be implemented in c++ to be
-                    # practical, and alternative is to interpolate samples
-                    # like hdr image generation and then naively add.
-                    sunsky, err = self.sunfield.query_ray(psi, skyrays)
-                    skysun, err = self.skyfield.query_ray(pi, sunrays)
-                    lmsun += skylm[skysun]
-                    lmsky += sunlm[sunsky]
-                    skyweight = .5
-
-                    fsun = [f(vm, sunrays, osun, lmsun, **kwargs) * skyweight
-                            for f in metricfuncs]
-                    mstack.append(fsun)
-                fsky = [f(vm, skyrays, osky, lmsky, **kwargs)*skyweight
-                        for f in metricfuncs]
-                mstack.append(fsky)
-                try:
-                    fsv = self.sunfield.view.metric(psi, vm, s, metricfuncs,
-                                                    **kwargs)
-                except (AttributeError, ValueError):
-                    pass
-                else:
-                    sunvalue += 1
-                    mstack.append(fsv)
-                smetrics.append(np.concatenate((np.sum(mstack, 0), [sunvalue])))
-            pmetrics.append(smetrics)
-        # loop returns shape: (points, skys, metrics)
-        # transpose reorganizes as: (view, points, skys, metrics)
-        return np.array(pmetrics)
+                for sj in range(len(smtx)):
+                    if dohdr:
+                        outf = (f"{self.scene.outdir}_{vname}_{pj:04d}_{sj:04d}"
+                                ".hdr")
+                        fu.append(exc.submit(self.hdr, sunsky, smtx[sj],
+                                             suns[sj], suni[sj], pi, vm, pdirs,
+                                             mask, vstr, outf, interp))
+                    if dometric:
+                        info = [[], []]
+                        infos0 = dict(idx=[pi, sj], sensor=[*pt, *vdir])
+                        infos1 = dict(err=[perr, sunerrs[sj]],
+                                      sky=[*skydata[sj]])
+                        for k, v in infos0.items():
+                            if metricreturn[k]:
+                                info[0] += v
+                        for k, v in infos1.items():
+                            if metricreturn[k]:
+                                info[1] += v
+                        fu.append(exc.submit(self.metric, sunsky, smtx[sj],
+                                             suns[sj], suni[sj], pi, vmm, info,
+                                             mfuncs, **kwargs))
+            outmetrics = []
+            for future in fu:
+                out, kind = future.result()
+                if kind == 'hdr':
+                    print(out, file=sys.stderr)
+                elif kind == 'metric':
+                    outmetrics.append(out)
+        if dometric:
+            headers = dict(
+                idx=["pt-idx", "sky-idx"],
+                sensor=["x", "y", "z", "dx", "dy", "dz"],
+                metric=[f.__name__ for f in mfuncs] + ["sun-value"],
+                err=["pt-err", "sun-err"],
+                sky=["sun-x", "sun-y", "sun-z", "dir-norm", "diff-horiz"])
+            colhdr = []
+            for k, v in headers.items():
+                if metricreturn[k]:
+                    colhdr += v
+            d = np.array(outmetrics)
+            cols = d.shape[1]
+            unsort = np.argsort(sort)
+            d = d.reshape(-1, len(smtx), cols)[unsort].reshape(-1, cols)
+            return colhdr, d
+        return None, []
