@@ -6,10 +6,6 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 # =======================================================================
 import os
-import pickle
-import concurrent.futures
-from concurrent.futures import ThreadPoolExecutor
-
 import numpy as np
 
 
