@@ -18,6 +18,7 @@ from raytraverse.lightfield.memarraydict import MemArrayDict
 
 from raytraverse import io, translate
 from raytraverse.lightfield.lightfield import LightField
+from raytraverse.craytraverse import interpolate_kdquery
 
 
 class LightFieldKD(LightField):
@@ -37,7 +38,7 @@ class LightFieldKD(LightField):
                 _voronoi.sort_vertices_of_regions(self._simplices, reg)
 
         d_kd = cKDTree(v)
-        omega = SVoronoi(v).calculate_areas()[:, None]
+        omega = SphericalVoronoi(v).calculate_areas()[:, None]
         return d_kd, omega
 
     @property
@@ -66,6 +67,10 @@ class LightFieldKD(LightField):
         kdfile = f'{scene.outdir}/{self.prefix}_kd_data.pickle'
         lumfile = f'{scene.outdir}/{self.prefix}_kd_lum_map.pickle'
         lumdat = f'{self.scene.outdir}/{self.prefix}_kd_lum.dat'
+        if self.rebuild and not all([os.path.isfile(p) for p in
+                                     self.raw_files()]):
+            raise FileNotFoundError("some data-files missing, cannot rebuild"
+                                    " lightfield")
         if (os.path.isfile(kdfile) and os.path.isfile(lumfile)
                 and not self.rebuild):
             f = open(kdfile, 'rb')
@@ -140,9 +145,9 @@ class LightFieldKD(LightField):
                 lummap.append((pt, lm.result()))
         return vecs, ltype(lummap)
 
-    def _mk_tree(self, pref='', ltype=MemArrayDict, os0=0):
+    def _mk_tree(self, pref='', ltype=MemArrayDict):
         npts = self.scene.area.npts
-        vs, lums = self._get_vl(npts, pref=pref, ltype=ltype, os0=os0)
+        vs, lums = self._get_vl(npts, pref=pref, ltype=ltype)
         with ProcessPoolExecutor(io.get_nproc()) as exc:
             d_kd, omega = zip(*exc.map(LightFieldKD.mk_vector_ball,
                                        vs.values()))
@@ -152,14 +157,14 @@ class LightFieldKD(LightField):
         c = np.asarray(coefs).reshape(-1, 1)
         return np.einsum('ij,kj->ik', c, self.lum[pi])
 
-    def add_to_img(self, img, mask, pi, i, d, coefs=1, vm=None, radius=3):
-        lum = self.apply_coef(pi, coefs)
-        if len(i.shape) > 1:
-            # gaussian reconstruction filter
-            y = norm(scale=translate.theta2chord(radius*np.pi/180))
-            w = np.broadcast_to(y.pdf(d), (lum.shape[0],) + d.shape)
-            lum = np.average(lum[:, i], weights=w, axis=-1)
+    def add_to_img(self, img, mask, pi, vecs, coefs=1, vm=None, interp=1,
+                   **kwargs):
+        if interp > 1:
+            arrout = self.interpolate_query(pi, vecs, k=interp, **kwargs)
+            lum = np.einsum('j,kj->k', coefs, arrout)
         else:
+            i, d = self.query_ray(pi, vecs)
+            lum = self.apply_coef(pi, coefs)
             lum = lum[:, i]
         img[mask] += np.squeeze(lum)
 
@@ -184,30 +189,54 @@ class LightFieldKD(LightField):
         vs = translate.theta2chord(viewangle/360*np.pi)
         return self.d_kd[pi].query_ball_point(translate.norm(vecs), vs)
 
+    def interpolate_query(self, pi, dest_vec, k=8,
+                          err=0.00436, up=0.347296):
+        """query a kd_tree and interpolate corresponding values. used to
+        merge to kd_trees with vector and luminance
+
+        Parameters
+        ----------
+        pi:
+            key
+        dest_vec: np.array
+            destination vectors to interpolate to, shape (N, 3)
+        k: int
+            initial query size
+        err: float
+            chord length under which value is taken without interpolation
+            default is .25 degrees = translate.theta2chord(.25*pi/180)
+        up: float
+            chord length of maximum search radius for neighbors
+            default is 10 degrees  = translate.theta2chord(20*pi/180)
+
+        Returns
+        -------
+        np.array
+            shape of (dest_vec.shape[0], src_lum.shape[1])
+        """
+        errs, idxs = self.d_kd[pi].query(dest_vec, k=k, distance_upper_bound=up)
+        if k == 1:
+            return self.lum[pi][idxs]
+        arrout = interpolate_kdquery(dest_vec, errs, idxs, self.vec[pi],
+                                     self.lum[pi], err=err)
+        return arrout
+
     def _dview(self, idx, pdirs, mask, res=800, showsample=True):
         img = np.zeros((res*self.scene.view.aspect, res))
-        i, d = self.query_ray(idx, pdirs[mask])
-        self.add_to_img(img, mask, idx, i, d)
+        self.add_to_img(img, mask, idx, pdirs[mask])
         outf = f"{self.outfile(idx)}.hdr"
         if showsample:
             vm = self.scene.view
             img = np.repeat(img[None, ...], 3, 0)
             vi = self.query_ball(idx, vm.dxyz, vm.viewangle)
             v = self.vec[idx][vi[0]]
-            reverse = vm.degrees(v) > 90
-            pa = vm.ivm.ray2pixel(v[reverse], res)
-            pa[:, 0] += res
-            pb = vm.ray2pixel(v[np.logical_not(reverse)], res)
-            xp = np.concatenate((pa[:, 0], pb[:, 0]))
-            yp = np.concatenate((pa[:, 1], pb[:, 1]))
-            img[1:, xp, yp] = 0
-            img[0, xp, yp] = 1
+            img = io.add_vecs_to_img(vm, img, v)
             io.carray2hdr(img, outf)
         else:
             io.array2hdr(img, outf)
         return outf
 
-    def direct_view(self, res=800, showsample=True, items=None):
+    def direct_view(self, res=512, showsample=True, items=None):
         """create a summary image of lightfield for each vpt"""
         vm = self.scene.view
         pdirs = vm.pixelrays(res)
