@@ -15,9 +15,10 @@ class SCBinField(LightFieldKD):
     each source
     """
 
-    def __init__(self, scene, rebuild=False, prefix='sky', **kwargs):
+    def __init__(self, scene, rebuild=False, prefix='sky', ground=True,
+                 **kwargs):
         super().__init__(scene, rebuild=rebuild, prefix=prefix,
-                         srcn=scene.skyres**2, **kwargs)
+                         srcn=scene.skyres**2 + ground, **kwargs)
 
     def apply_coef(self, pi, coefs):
         coefs = np.asarray(coefs)
