@@ -5,18 +5,12 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 # =======================================================================
-import os
-import shutil
-
 from raytraverse.renderer.radiancerenderer import RadianceRenderer
 from raytraverse.crenderer import cRcontrib
-import raytraverse
-
-path = os.path.dirname(raytraverse.__file__)
-rcontrib_capture_file = f'{path}/tmp_rcontrib_capture'
 
 if cRcontrib.version == "PyVirtual":
-    from raytraverse.renderer.sprenderer import SPRcontrib as Rcontrib
+    from raytraverse.renderer.sprenderer import SPRcontrib
+    Rcontrib = SPRcontrib
 else:
     class Rcontrib(RadianceRenderer):
         """singleton wrapper for c++ crenderer.cRcontrib singleton class"""
