@@ -50,6 +50,13 @@ class SPRenderer(Renderer):
             q.communicate()
         return capture.stdout
 
+    @classmethod
+    def _set_args(cls, args, iot, nproc):
+        print((f"{cls.name} -f{iot} -n {nproc} {cls.arg_prefix}"
+                           f" {args}"))
+        return shlex.split(f"{cls.name} -f{iot} -n {nproc} {cls.arg_prefix}"
+                           f" {args}")
+
 
 class SPRtrace(SPRenderer):
     Engine = 'rtrace'
