@@ -91,7 +91,7 @@ class SingleSunSampler(Sampler):
                 pass
         super().__del__()
 
-    def pdf_from_sky(self, skyfield, interp=12, rebuild=False, zero=True,
+    def pdf_from_sky(self, skyfield, interp=1, rebuild=False, zero=True,
                      filterpts=True):
         ishape = np.concatenate((self.scene.area.ptshape,
                                  self.levels[self.specidx-2]))
@@ -107,19 +107,22 @@ class SingleSunSampler(Sampler):
             grid = skyfield.scene.view.uv2xyz(uv)
             idxs, errs = skyfield.query_all_pts(grid, interp)
             strides = np.array(skyfield.lum.index_strides()[:-1])[:, None, None]
-            idxs = np.reshape(idxs + strides, (-1, interp))
+            idxs = np.reshape(np.atleast_3d(idxs) + strides, (-1, interp))
             errs = errs.reshape(-1, interp)
             np.savez(fi, idxs, errs)
-        column = skyfield.lum.full_array()[:, self.sbin]
+        column = skyfield.lum.full_array()
+        column = column[:, self.sbin]
+        lum = column[idxs]
+        if zero:
+            lum = np.where(lum > self.scene.maxspec, 0, lum)
         if interp > 1:
-            lum = np.average(column[idxs], -1, weights=1/errs).reshape(ishape)
+            lum = np.average(lum, -1, weights=1/errs).reshape(ishape)
         else:
-            lum = column[idxs].reshape(ishape)
+            lum = lum.reshape(ishape)
         if filterpts:
             haspeak = np.max(lum, (2, 3)) > self.srct
             lum = lum * haspeak[..., None, None]
-        if zero:
-            lum = np.where(lum > self.scene.maxspec, 0, lum)
+
         return lum
 
     def sample(self, vecf):
@@ -156,7 +159,7 @@ class SingleSunSampler(Sampler):
             weights = self.pdf_from_sky(skyfield)
             p = translate.resample(weights, shape)
             if self.plotp:
-                quickplot.imshow(p[0, 0], [20, 10])
+                self._plot_p(p)
             pdraws = draw.from_pdf(p.ravel(), self.slimit)
         else:
             pdraws = super().draw()
